@@ -17,6 +17,7 @@ from typing import (
     Callable, Any, Union, Dict, List, TypeVar, Tuple, Set, Sequence, get_type_hints, TextIO,
     Optional, IO, BinaryIO, Type, Generator, overload, Iterable, AsyncIterable, Iterator,
     AsyncIterator, AbstractSet)
+from unittest.mock import Mock
 from warnings import warn
 from weakref import WeakKeyDictionary, WeakValueDictionary
 
@@ -26,9 +27,9 @@ except ImportError:
     Literal = TypedDict = None
 
 try:
-    from typing import Protocol
+    from typing_extensions import Literal as BPLiteral
 except ImportError:
-    from typing import _Protocol as Protocol
+    BPLiteral = None
 
 try:
     from typing import AsyncGenerator
@@ -163,7 +164,7 @@ def resolve_forwardref(maybe_ref, memo: _CallMemo):
 
 def get_type_name(type_):
     # typing.* types don't have a __name__ on Python 3.7+
-    return getattr(type_, '__name__', None) or type_._name
+    return getattr(type_, '__name__', None) or type_._name or str(type_)
 
 
 def find_function(frame) -> Optional[Callable]:
@@ -432,10 +433,10 @@ def check_class(argname: str, value, expected_type, memo: Optional[_CallMemo]) -
 
 def check_typevar(argname: str, value, typevar: TypeVar, memo: Optional[_CallMemo],
                   subclass_check: bool = False) -> None:
-    if memo is None:
-        raise TypeError('encountered a TypeVar but a call memo was not provided')
+    bound_type = None
+    if memo is not None:
+        bound_type = resolve_forwardref(memo.typevars.get(typevar, typevar.__bound__), memo)
 
-    bound_type = resolve_forwardref(memo.typevars.get(typevar, typevar.__bound__), memo)
     value_type = value if subclass_check else type(value)
     subject = argname if subclass_check else 'type of ' + argname
     if bound_type is None:
@@ -463,15 +464,21 @@ def check_typevar(argname: str, value, typevar: TypeVar, memo: Optional[_CallMem
                 '{} must be exactly {}; got {} instead'.
                 format(subject, qualified_name(bound_type), qualified_name(value_type)))
 
-    if typevar not in memo.typevars:
+    if memo is not None and typevar not in memo.typevars:
         # Bind the type variable to a concrete type
         memo.typevars[typevar] = value_type
 
 
 def check_literal(argname: str, value, expected_type, memo: Optional[_CallMemo]):
-    if value not in expected_type.__args__:
+    try:
+        args = expected_type.__args__
+    except AttributeError:
+        # Instance of Literal from typing_extensions
+        args = expected_type.__values__
+
+    if value not in args:
         raise TypeError('the value of {} must be one of {}; got {} instead'.
-                        format(argname, expected_type.__args__, value))
+                        format(argname, args, value))
 
 
 def check_number(argname: str, value, expected_type):
@@ -527,6 +534,8 @@ if Type is not None:
     origin_type_checkers[Type] = check_class
 if Literal is not None:
     origin_type_checkers[Literal] = check_literal
+if BPLiteral is not None:
+    origin_type_checkers[BPLiteral] = check_literal
 
 generator_origin_types = (Generator, collections.abc.Generator,
                           Iterator, collections.abc.Iterator,
@@ -552,7 +561,7 @@ def check_type(argname: str, value, expected_type, memo: Optional[_CallMemo] = N
     :param expected_type: a class or generic type instance
 
     """
-    if expected_type is Any:
+    if expected_type is Any or isinstance(value, Mock):
         return
 
     if expected_type is None:
@@ -599,6 +608,9 @@ def check_type(argname: str, value, expected_type, memo: Optional[_CallMemo] = N
     elif isinstance(expected_type, TypeVar):
         # Only happens on < 3.6
         check_typevar(argname, value, expected_type, memo)
+    elif BPLiteral is not None and isinstance(expected_type, BPLiteral.__class__):
+        # Only happens on < 3.7 when using Literal from typing_extensions
+        check_literal(argname, value, expected_type, memo)
     elif (isfunction(expected_type) and
             getattr(expected_type, "__module__", None) == "typing" and
             getattr(expected_type, "__qualname__", None).startswith("NewType.") and
@@ -813,11 +825,12 @@ def typechecked(func=None, *, always=False, _localns: Optional[Dict[str, Any]] =
         # If a generator is returned, wrap it if its yield/send/return types can be checked
         if inspect.isgenerator(retval) or isasyncgen(retval):
             return_type = memo.type_hints.get('return')
-            origin = getattr(return_type, '__origin__')
-            if origin in generator_origin_types:
-                return TypeCheckedGenerator(retval, memo)
-            elif origin is not None and origin in asyncgen_origin_types:
-                return TypeCheckedAsyncGenerator(retval, memo)
+            if return_type:
+                origin = getattr(return_type, '__origin__', None)
+                if origin in generator_origin_types:
+                    return TypeCheckedGenerator(retval, memo)
+                elif origin is not None and origin in asyncgen_origin_types:
+                    return TypeCheckedAsyncGenerator(retval, memo)
 
         return retval
 
